/*
 * Copyright (c) 2019 Helmut Tschemernjak
 * 30826 Garbsen (Hannover) Germany
 * Licensed under the Apache License, Version 2.0);
 */

 /*
  * TODO:
  * Compiler Date/Time is not set correctly on startup
  */
#include "main.h"
#include "RadioShuttle.h"
#include "RadioTest.h"
#include "GenericPingPong.h"
#ifdef FEATURE_NVPROPERTY
#include <NVPropertyProviderInterface.h>
#include "NVProperty.h"
#endif

DigitalOut statusLED(LED);
DigitalOut redLED(LED2);
InterruptIn buttonIntr(USER_BUTTON);
volatile int pressedCount;

void switchInput(void) {
	InterrruptMSG(INT_BUTTON1);
}


void timerUpdate(void) {
	static LowPowerTimeout timeout;
	if (redLED == 0)
		timeout.attach_us(&timerUpdate, 20000); // setup to call timerUpdate after 20 millis
	else
    	timeout.attach_us(&timerUpdate, 2000000); // setup to call timerUpdate after 2 seconds

	InterrruptMSG(INT_TIMEOUT);
}


int main() {
    /*
     * inits the Serial or USBSerial when available (230400 baud).
     * If the serial uart is not is not connected it swiches to USB Serial
     * blinking LED means USBSerial detected, waiting for a connect.
     * It waits up to 30 seconds for a USB terminal connections 
     */
    InitSerial(30*1000, &statusLED);
	dprintf("Welcome to RadioShuttle v%d.%d", RS_MAJOR, RS_MINOR);
    dprintf("Voltage: %.2f (%s powered)", BatteryVoltage(), BatterySource());
	
	char buf[32];
	
	bool cmdLoop = true;
	while(cmdLoop) {
		rprintf("\r\nAvailable commands (p = Property Editor, r = Reset, q = Quit)\r\nwaiting 10 secs ...\r\n");
		rprintf("\r\n$ ");
		if (ConsoleReadline(buf, sizeof(buf), true, 10000) == NULL) {
			cmdLoop = false;
			break;
		}
		switch(buf[0]) {
			case 'p':
			case 'P':
#ifdef FEATURE_NVPROPERTYEDITOR
				NVPropertyEditor();
#endif
			break;
			case 'q':
			case 'Q':
			cmdLoop = false;
			break;
			case 'r':
			case 'R':
				MCUReset();
				break;
			default:
			break;
		}
	}
	
#ifdef FEATURE_LORA
    InitRadio();
#endif
#ifdef FEATURE_LORA_PING_PONG
    DeInitRadio();
    SX1276PingPong();	// basic LoRa raw ping/pong without RadioShuttle
#endif
    dprintf("InitDefaults Done");
    timerUpdate(); // start timer
#if defined (USER_BUTTON_RISE) // attach switchInput function to the rising or falling edge
	buttonIntr.rise(&switchInput);
#else
	buttonIntr.fall(&switchInput);
#endif

	/*
	 * Main event loop, process interrupts and go to sleep
	 * the green statusLED indicates CPU activity
	 * the red redLED indicates that low power timeout function is running.
	 */
	while(true) {
        while ((readPendingInterrupts() == 0)) {
			statusLED = 0;
			if (!(usb && usb->connected()))
				sleep();
			statusLED = 1;
        }

        uint32_t pendirqs = readclrPendingInterrupts();
        if (pendirqs & INT_BUTTON1) {
#ifdef FEATURE_LORA
			statusLED = !statusLED;
            RadioUpdate(true); // pass the pressed user button to RadioShuttle
#endif
		}
        if (pendirqs & INT_LORA) {
#ifdef FEATURE_LORA
            RadioUpdate(false);
#endif
        }
        if (pendirqs & INT_TIMEOUT) {
			redLED = ! redLED;
		}
	}
}
